@@ -13,8 +13,8 @@ if(class(frame)=="try-error") stop("the formula inputted contains an error, e.g
 #### Design matrix
 ## Create the matrix
 X <- try(suppressWarnings(model.matrix(object=attr(frame, "terms"), data=frame)), silent=TRUE)
-    if(class(X)=="try-error") stop("the covariate matrix contains inappropriate values.", call.=FALSE)
-    if(sum(is.na(X))>0) stop("the covariate matrix contains missing 'NA' values.", call.=FALSE)
+     if(class(X)=="try-error") stop("the covariate matrix contains inappropriate values.", call.=FALSE)
+     if(sum(is.na(X))>0) stop("the covariate matrix contains missing 'NA' values.", call.=FALSE)
 
 n <- nrow(X)
 p <- ncol(X)
@@ -28,7 +28,7 @@ diag(cor.X) <- 0
 
 	 if(p>1)
 	 {
-    	if(sort(apply(X, 2, sd))[2]==0) stop("the covariate matrix has two intercept terms.", call.=FALSE)
+    	 if(sort(apply(X, 2, sd))[2]==0) stop("the covariate matrix has two intercept terms.", call.=FALSE)
 	 }else
 	 {
 	 }
@@ -92,7 +92,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 ## Regression parameters beta
 dat <- cbind(Y, trials-Y)
 
-	 if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
+    if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
     if(length(beta)!= p) stop("beta is the wrong length.", call.=FALSE)
     if(sum(is.na(beta))>0) stop("beta has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(beta)) stop("beta has non-numeric values.", call.=FALSE)
@@ -136,13 +136,69 @@ dat <- cbind(Y, trials-Y)
     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.beta)==ceiling(blocksize.beta))) stop("blocksize.beta has non-integer values.", call.=FALSE)
-     if(!is.numeric(blocksize.phi)) stop("blocksize.phi is not a number", call.=FALSE)
+    if(!is.numeric(blocksize.phi)) stop("blocksize.phi is not a number", call.=FALSE)
     if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
-     if(!is.numeric(blocksize.theta)) stop("blocksize.theta is not a number", call.=FALSE)
+    if(!is.numeric(blocksize.theta)) stop("blocksize.theta is not a number", call.=FALSE)
     if(blocksize.theta <= 0) stop("blocksize.theta is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.theta)==ceiling(blocksize.theta))) stop("blocksize.theta has non-integer values.", call.=FALSE)
 
+
+## Compute the blocking structure for beta
+     if(blocksize.beta >= p)
+     {
+     n.beta.block <- 1
+     beta.beg <- 1
+     beta.fin <- p
+     }else
+     {
+     n.standard <- 1 + floor((p-blocksize.beta) / blocksize.beta)
+     remainder <- p - n.standard * blocksize.beta
+     
+          if(remainder==0)
+          {
+          beta.beg <- c(1,seq((blocksize.beta+1), p, blocksize.beta))
+          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta))
+          n.beta.block <- length(beta.beg)
+          }else
+          {
+          beta.beg <- c(1, seq((blocksize.beta+1), p, blocksize.beta))
+          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta), p)
+          n.beta.block <- length(beta.beg)
+          }
+     }         
+
+
+## Compute the blocking structure for phi
+     if(blocksize.phi >= n)
+     {
+     n.phi.block <- 1
+     phi.beg <- 1
+     phi.fin <- n  
+     }else
+     {
+     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
+     remainder <- n - (n.standard * blocksize.phi)
+     
+          if(remainder==0)
+          {
+          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
+          n.phi.block <- length(phi.beg)
+          }else if(remainder==1)
+          {
+          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)    
+          }else
+          {
+          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)
+          }
+     }
+
+
 ## Matrices to store samples
 samples.beta <- array(NA, c((n.sample-burnin), p))
 samples.phi <- array(NA, c((n.sample-burnin), n))
@@ -158,6 +214,9 @@ proposal.sd.beta <- 0.01
 proposal.sd.phi <- 0.1
 proposal.sd.theta <- 0.1
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
+chol.proposal.corr.beta <- chol(proposal.corr.beta) 
+tau2.posterior.shape <- 0.5 * n - 1
+sigma2.posterior.shape <- 0.5 * n - 1
 
 
 #### Priors
@@ -202,6 +261,18 @@ n.neighbours <- as.numeric(apply(W, 1, sum))
 Q <- diag(n.neighbours)  - W
 
 
+## quantities required in updating phi              
+block.mean.part <- as.list(rep(0,n.phi.block))
+block.var.chol <- as.list(rep(0,n.phi.block))
+
+     for(r in 1:n.phi.block)
+     {
+     Q.current <- Q[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
+     block.var <- chol2inv(chol(Q.current))
+     block.mean.part[[r]] <- - block.var %*% Q[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])]
+     block.var.chol[[r]] <- chol(block.var)
+     }
+
 
 #### Other quantities needed for the MCMC algorithm
 failures <- trials - Y
@@ -216,135 +287,71 @@ failures <- trials - Y
     ####################
     ## Sample from beta
     ####################
-    #### Create the blocking structure
-    if(blocksize.beta >= p)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- p
-    }else
-    {
-    init <- sample(1:blocksize.beta,  1)
-    n.standard <- floor((p-init) / blocksize.beta)
-    remainder <- p - (init + n.standard * blocksize.beta)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,p)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), p, blocksize.beta))
-        fin <- c(init, seq((init+blocksize.beta), p, blocksize.beta))
-        }else
-        {
-        beg <- c(1, seq((init+1), p, blocksize.beta))
-        fin <- c(init, seq((init+blocksize.beta), p, blocksize.beta), p)
-        }
-    n.block <- length(beg)
-    }
-    
-
-    #### Update the parameters in blocks
+    proposal <- beta + (sqrt(proposal.sd.beta)* t(chol.proposal.corr.beta)) %*% rnorm(p)
     proposal.beta <- beta    
-        
-        for(r in 1:n.block)
-        {
-        ## Propose a value
-        n.current <- length(beg[r]:fin[r])
-        proposal.beta[beg[r]:fin[r]] <- mvrnorm(n=1, mu=beta[beg[r]:fin[r]], Sigma=(proposal.sd.beta * proposal.corr.beta[beg[r]:fin[r], beg[r]:fin[r]]))
-        logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + phi + theta + offset
-        logit.current <- as.numeric(X.standardised %*% beta) + phi + theta + offset    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y * (log(prob.proposal) - log(prob.current)) + failures * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- sum(((beta[beg[r]:fin[r]] - prior.mean.beta[beg[r]:fin[r]])^2 - (proposal.beta[beg[r]:fin[r]] - prior.mean.beta[beg[r]:fin[r]])^2) / (2 * prior.var.beta[beg[r]:fin[r]]))
-        prob <- exp(prob1 + prob2)
-
-        ## Accept or reject the value
-            if(prob > runif(1))
-            {
-            beta <- proposal.beta
-            accept[1] <- accept[1] + 1  
-            accept[2] <- accept[2] + 1 
-            }else
-            {
-            proposal.beta <- beta
-            accept[2] <- accept[2] + 1 
-            }
-        }
-
+    phi.offset <- phi + theta + offset
+         
+         for(r in 1:n.beta.block)
+         {
+         ## Propose a value
+         proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
+         logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + phi.offset
+         logit.current <- as.numeric(X.standardised %*% beta) + phi.offset    
+         prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
+         prob.current <- exp(logit.current)  / (1 + exp(logit.current))
+              
+         ## Calculate the acceptance probability
+         prob1 <- sum(Y * (log(prob.proposal) - log(prob.current)) + failures * (log(1-prob.proposal) - log(1-prob.current)))          
+         prob2 <- sum(((beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2 - (proposal.beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2) / prior.var.beta[beta.beg[r]:beta.fin[r]])
+         prob <- exp(prob1 + prob2)
+              
+              ## Accept or reject the value
+              if(prob > runif(1))
+              {
+              beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
+              accept[1] <- accept[1] + 1  
+              }else
+              {
+              proposal.beta[beta.beg[r]:beta.fin[r]] <- beta[beta.beg[r]:beta.fin[r]]
+              }
+         }
+    accept[2] <- accept[2] + n.beta.block    
+         
 
 
     ####################
     ## Sample from phi
     ####################
-    #### Create the blocking structure
-    if(blocksize.phi >= n)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- n
-    }else
-    {
-    init <- sample(1:blocksize.phi,  1)
-    n.standard <- floor((n-init) / blocksize.phi)
-    remainder <- n - (init + n.standard * blocksize.phi)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,n)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi))
-        }else
-        {
-        beg <- c(1, seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi), n)
-        }
-    n.block <- length(beg)
-    }
-    
-
-    #### Update the parameters in blocks
     Q.temp <- Q / tau2
-    offset.temp <- as.numeric(X.standardised %*% beta) + theta + offset
-    proposal.phi <- phi    
-        
-        for(r in 1:n.block)
-        {
-        ## Propose a value
-        Q.current <- Q.temp[beg[r]:fin[r], beg[r]:fin[r]]
-        block.var <- chol2inv(chol(Q.current))
-        block.mean <- - block.var %*% Q.temp[beg[r]:fin[r], -(beg[r]:fin[r])] %*% phi[-(beg[r]:fin[r])]
-        proposal.phi[beg[r]:fin[r]] <- mvrnorm(n=1, mu=phi[beg[r]:fin[r]], Sigma=(proposal.sd.phi * block.var))
-        logit.proposal <- offset.temp[beg[r]:fin[r]] + proposal.phi[beg[r]:fin[r]]
-        logit.current <- offset.temp[beg[r]:fin[r]] + phi[beg[r]:fin[r]]    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y[beg[r]:fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[beg[r]:fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- t(phi[beg[r]:fin[r]] - block.mean) %*% Q.current %*% (phi[beg[r]:fin[r]] - block.mean) - t(proposal.phi[beg[r]:fin[r]] - block.mean) %*% Q.current %*% (proposal.phi[beg[r]:fin[r]] - block.mean)
-        prob <- exp(prob1 + 0.5 * prob2)
-
-        ## Accept or reject the value
-            if(prob > runif(1))
-            {
-            phi[beg[r]:fin[r]] <- proposal.phi[beg[r]:fin[r]]
-            accept[3] <- accept[3] + 1  
-            accept[4] <- accept[4] + 1 
-            }else
-            {
-            proposal.phi[beg[r]:fin[r]] <- phi[beg[r]:fin[r]]
-            accept[4] <- accept[4] + 1 
-            }
-        }
-        
+    beta.offset <- as.numeric(X.standardised %*% beta) + theta + offset        
+    b <- rnorm(n)
+    
+         for(r in 1:n.phi.block)
+         {
+         ## Propose a value
+         Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
+         block.mean <- block.mean.part[[r]] %*% phi[-(phi.beg[r]:phi.fin[r])]
+         proposal.phi <- phi[phi.beg[r]:phi.fin[r]] + (sqrt(proposal.sd.phi) * sqrt(tau2) * t(block.var.chol[[r]])) %*% b[phi.beg[r]:phi.fin[r]]
+         logit.proposal <- beta.offset[phi.beg[r]:phi.fin[r]] + proposal.phi
+         logit.current <- beta.offset[phi.beg[r]:phi.fin[r]] + phi[phi.beg[r]:phi.fin[r]]    
+         prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
+         prob.current <- exp(logit.current)  / (1 + exp(logit.current))
+         
+         ## Calculate the acceptance probability
+         prob1 <- sum(Y[phi.beg[r]:phi.fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[phi.beg[r]:phi.fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
+         prob2 <- t(phi[phi.beg[r]:phi.fin[r]] - block.mean) %*% Q.current %*% (phi[phi.beg[r]:phi.fin[r]] - block.mean) - t(proposal.phi - block.mean) %*% Q.current %*% (proposal.phi - block.mean)
+         prob <- exp(prob1 + 0.5 * prob2)
+         
+         ## Accept or reject the value
+              if(prob > runif(1))
+              {
+              phi[phi.beg[r]:phi.fin[r]] <- proposal.phi
+              accept[3] <- accept[3] + 1  
+              }else
+              {
+              }
+         }                      
+    accept[4] <- accept[4] + n.phi.block
     phi <- phi - mean(phi)
     
     
@@ -352,90 +359,47 @@ failures <- trials - Y
     ####################
     ## Sample from theta
     ####################
-    #### Create the blocking structure
-    if(blocksize.theta >= n)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- n
-    }else
-    {
-    init <- sample(1:blocksize.theta,  1)
-    n.standard <- floor((n-init) / blocksize.theta)
-    remainder <- n - (init + n.standard * blocksize.theta)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,n)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), n, blocksize.theta))
-        fin <- c(init, seq((init+blocksize.theta), n, blocksize.theta))
-        }else
-        {
-        beg <- c(1, seq((init+1), n, blocksize.theta))
-        fin <- c(init, seq((init+blocksize.theta), n, blocksize.theta), n)
-        }
-    n.block <- length(beg)
-    }
+    proposal.theta <- rnorm(n, mean=theta, sd=proposal.sd.theta)
+    beta.offset <- as.numeric(X.standardised %*% beta) + phi + offset        
+    logit.proposal <- beta.offset + proposal.theta
+    logit.current <- beta.offset + theta    
+    prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
+    prob.current <- exp(logit.current)  / (1 + exp(logit.current))
     
-
-    #### Update the parameters in blocks
-    offset.temp <- as.numeric(X.standardised %*% beta) + phi + offset
-    proposal.theta <- theta    
-        
-        for(r in 1:n.block)
-        {
-        ## Propose a value
-        n.current <- length(beg[r]:fin[r])
-        proposal.theta[beg[r]:fin[r]] <- rnorm(n=n.current, mean=theta[beg[r]:fin[r]], sd=rep(proposal.sd.theta, n.current))
-        logit.proposal <- offset.temp[beg[r]:fin[r]] + proposal.theta[beg[r]:fin[r]]
-        logit.current <- offset.temp[beg[r]:fin[r]] + theta[beg[r]:fin[r]]    
-        prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
-        prob.current <- exp(logit.current)  / (1 + exp(logit.current))
-
-        ## Calculate the acceptance probability
-        prob1 <- sum(Y[beg[r]:fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[beg[r]:fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- sum(theta[beg[r]:fin[r]]^2  - proposal.theta[beg[r]:fin[r]]^2) / (2 * sigma2)
-        prob <- exp(prob1 + prob2)
-
-        ## Accept or reject the value
-            if(prob > runif(1))
-            {
-            theta[beg[r]:fin[r]] <- proposal.theta[beg[r]:fin[r]]
-            accept[5] <- accept[5] + 1  
-            accept[6] <- accept[6] + 1 
-            }else
-            {
-            proposal.theta[beg[r]:fin[r]] <- theta[beg[r]:fin[r]]
-            accept[6] <- accept[6] + 1 
-            }
-        }
+         for(r in 1:n.phi.block)
+         {
+         ## Calculate the acceptance probability
+         prob1 <- sum(Y[phi.beg[r]:phi.fin[r]] * (log(prob.proposal[phi.beg[r]:phi.fin[r]]) - log(prob.current[phi.beg[r]:phi.fin[r]])) + failures[phi.beg[r]:phi.fin[r]] * (log(1-prob.proposal[phi.beg[r]:phi.fin[r]]) - log(1-prob.current[phi.beg[r]:phi.fin[r]])))          
+         prob2 <- sum(theta[phi.beg[r]:phi.fin[r]]^2  - proposal.theta[phi.beg[r]:phi.fin[r]]^2) / sigma2
+         prob <- exp(prob1 + 0.5 * prob2)
+         
+         ## Accept or reject the value
+              if(prob > runif(1))
+              {
+              theta[phi.beg[r]:phi.fin[r]] <- proposal.theta[phi.beg[r]:phi.fin[r]]
+              accept[5] <- accept[5] + 1  
+              }else
+              {
+              }
+         }
+    accept[6] <- accept[6] + n.phi.block
     theta <- theta - mean(theta)
-
+    
 
 
     ###################
     ## Sample from tau2
     ###################
-    tau2.posterior.scale <- 0.5 * t(phi) %*% Q %*% phi
-    tau2 <- rinvgamma(n=1, shape=(0.5*(n-3)), scale=tau2.posterior.scale)
-            while(tau2 > prior.max.tau2)
-            {
-            tau2 <- rinvgamma(n=1, shape=(0.5*(n-3)), scale=tau2.posterior.scale)
-            }
+    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
+    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
     
     
     #####################
     ## Sample from sigma2
     #####################
-    sigma2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=(0.5*sum(theta^2)))
-            while(sigma2 > prior.max.sigma2)
-            {
-            sigma2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=(0.5*sum(theta^2)))
-            }
+    sigma2.posterior.scale <- 0.5*sum(theta^2)
+    sigma2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.sigma2), b=Inf,  shape=sigma2.posterior.shape, scale=(1/sigma2.posterior.scale))
     
     
     
@@ -444,7 +408,7 @@ failures <- trials - Y
     #########################
     logit <- as.numeric(X.standardised %*% beta) + phi + theta + offset    
     prob <- exp(logit)  / (1 + exp(logit))
-	 deviance <- -2 * sum(dbinom(x=Y, size=trials, prob=prob, log=TRUE))
+    deviance <- -2 * sum(dbinom(x=Y, size=trials, prob=prob, log=TRUE))
 
 
 
