@@ -92,7 +92,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 ## Regression parameters beta
 dat <- cbind(Y, trials-Y)
 
-	 if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
+    if(is.null(beta)) beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
     if(length(beta)!= p) stop("beta is the wrong length.", call.=FALSE)
     if(sum(is.na(beta))>0) stop("beta has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(beta)) stop("beta has non-numeric values.", call.=FALSE)
@@ -134,6 +134,62 @@ dat <- cbind(Y, trials-Y)
     if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
 
+
+## Compute the blocking structure for beta
+     if(blocksize.beta >= p)
+     {
+     n.beta.block <- 1
+     beta.beg <- 1
+     beta.fin <- p
+     }else
+     {
+     n.standard <- 1 + floor((p-blocksize.beta) / blocksize.beta)
+     remainder <- p - n.standard * blocksize.beta
+     
+          if(remainder==0)
+          {
+          beta.beg <- c(1,seq((blocksize.beta+1), p, blocksize.beta))
+          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta))
+          n.beta.block <- length(beta.beg)
+          }else
+          {
+          beta.beg <- c(1, seq((blocksize.beta+1), p, blocksize.beta))
+          beta.fin <- c(blocksize.beta, seq((blocksize.beta+blocksize.beta), p, blocksize.beta), p)
+          n.beta.block <- length(beta.beg)
+          }
+     }         
+
+
+## Compute the blocking structure for phi
+     if(blocksize.phi >= n)
+     {
+     n.phi.block <- 1
+     phi.beg <- 1
+     phi.fin <- n  
+     }else
+     {
+     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
+     remainder <- n - (n.standard * blocksize.phi)
+     
+          if(remainder==0)
+          {
+          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
+          n.phi.block <- length(phi.beg)
+          }else if(remainder==1)
+          {
+          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)    
+          }else
+          {
+          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)
+          }
+     }
+
+
 ## Matrices to store samples
 samples.beta <- array(NA, c((n.sample-burnin), p))
 samples.phi <- array(NA, c((n.sample-burnin), n))
@@ -142,12 +198,14 @@ samples.rho <- array(NA, c((n.sample-burnin), 1))
 samples.deviance <- array(NA, c((n.sample-burnin), 1))
 
 ## Metropolis quantities
-accept.all <- rep(0,6)
+accept.all <- rep(0,4)
 accept <- accept.all
 proposal.sd.beta <- 0.01
 proposal.sd.phi <- 0.1
-proposal.sd.rho <- 0.05
+proposal.sd.rho <- 0.02
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
+chol.proposal.corr.beta <- chol(proposal.corr.beta) 
+tau2.posterior.shape <- 0.5 * n - 1
 
 
 #### Priors
@@ -183,11 +241,27 @@ proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
     if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
     if(!sum(names(table(W))==c(0,1))==2) stop("W has non-binary (zero and one) values.", call.=FALSE)
 
-n.neighbours <- as.numeric(apply(W, 1, sum))
-Q <- -rho * W
-diag(Q) <- n.neighbours * rho + 1 - rho
-det.Q <- 0.5 * as.numeric(determinant(Q, logarithm=TRUE))[1]
 
+## Create the precision matrix Q
+n.neighbours <- as.numeric(apply(W, 1, sum))
+Wstar <- diag(n.neighbours) - W
+Wstar.eigen <- eigen(Wstar)
+Wstar.val <- diag(Wstar.eigen$values)
+Wstar.vec <- Wstar.eigen$vectors
+Q <- rho * Wstar + (1-rho) * diag(rep(1,n))
+det.Q <- 0.5 * sum(log((rho * diag(Wstar.val) + (1-rho))))
+
+
+## Calculate eigen decomposition for blocks of Wstar               
+Wstar.list.vectors <- as.list(rep(0,n.phi.block))
+Wstar.list.values <- as.list(rep(0,n.phi.block))
+
+     for(i in 1:n.phi.block)
+     {
+     temp <- eigen(Wstar[phi.beg[i]:phi.fin[i], phi.beg[i]:phi.fin[i]])     
+     Wstar.list.vectors[[i]] <- temp$vectors
+     Wstar.list.values[[i]] <- temp$values
+     }
 
 
 #### Other quantities needed for the MCMC algorithm
@@ -203,148 +277,83 @@ failures <- trials - Y
     ####################
     ## Sample from beta
     ####################
-    #### Create the blocking structure
-    if(blocksize.beta >= p)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- p
-    }else
-    {
-    init <- sample(1:blocksize.beta,  1)
-    n.standard <- floor((p-init) / blocksize.beta)
-    remainder <- p - (init + n.standard * blocksize.beta)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,p)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), p, blocksize.beta))
-        fin <- c(init, seq((init+blocksize.beta), p, blocksize.beta))
-        }else
-        {
-        beg <- c(1, seq((init+1), p, blocksize.beta))
-        fin <- c(init, seq((init+blocksize.beta), p, blocksize.beta), p)
-        }
-    n.block <- length(beg)
-    }
-    
-
-    #### Update the parameters in blocks
+    proposal <- beta + (sqrt(proposal.sd.beta)* t(chol.proposal.corr.beta)) %*% rnorm(p)
     proposal.beta <- beta    
+    phi.offset <- phi + offset
         
-        for(r in 1:n.block)
+        for(r in 1:n.beta.block)
         {
         ## Propose a value
-        n.current <- length(beg[r]:fin[r])
-        proposal.beta[beg[r]:fin[r]] <- mvrnorm(n=1, mu=beta[beg[r]:fin[r]], Sigma=(proposal.sd.beta * proposal.corr.beta[beg[r]:fin[r], beg[r]:fin[r]]))
-        logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + phi + offset
-        logit.current <- as.numeric(X.standardised %*% beta) + phi + offset    
+        proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
+        logit.proposal <- as.numeric(X.standardised %*% proposal.beta) + phi.offset
+        logit.current <- as.numeric(X.standardised %*% beta) + phi.offset    
         prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
         prob.current <- exp(logit.current)  / (1 + exp(logit.current))
 
         ## Calculate the acceptance probability
         prob1 <- sum(Y * (log(prob.proposal) - log(prob.current)) + failures * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- sum(((beta[beg[r]:fin[r]] - prior.mean.beta[beg[r]:fin[r]])^2 - (proposal.beta[beg[r]:fin[r]] - prior.mean.beta[beg[r]:fin[r]])^2) / (2 * prior.var.beta[beg[r]:fin[r]]))
+        prob2 <- sum(((beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2 - (proposal.beta[beta.beg[r]:beta.fin[r]] - prior.mean.beta[beta.beg[r]:beta.fin[r]])^2) / prior.var.beta[beta.beg[r]:beta.fin[r]])
         prob <- exp(prob1 + prob2)
 
         ## Accept or reject the value
             if(prob > runif(1))
             {
-            beta <- proposal.beta
+            beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
             accept[1] <- accept[1] + 1  
-            accept[2] <- accept[2] + 1 
             }else
             {
-            proposal.beta <- beta
-            accept[2] <- accept[2] + 1 
+            proposal.beta[beta.beg[r]:beta.fin[r]] <- beta[beta.beg[r]:beta.fin[r]]
             }
         }
 
-
+    accept[2] <- accept[2] + n.beta.block    
+    
+    
 
     ####################
     ## Sample from phi
     ####################
-    #### Create the blocking structure
-    if(blocksize.phi >= n)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- n
-    }else
-    {
-    init <- sample(1:blocksize.phi,  1)
-    n.standard <- floor((n-init) / blocksize.phi)
-    remainder <- n - (init + n.standard * blocksize.phi)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,n)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi))
-        }else
-        {
-        beg <- c(1, seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi), n)
-        }
-    n.block <- length(beg)
-    }
-    
-
-    #### Update the parameters in blocks
     Q.temp <- Q / tau2
-    beta.offset <- as.numeric(X.standardised %*% beta) + offset
-    proposal.phi <- phi    
-        
-        for(r in 1:n.block)
+    beta.offset <- as.numeric(X.standardised %*% beta) + offset        
+    b <- rnorm(n)
+         
+        for(r in 1:n.phi.block)
         {
         ## Propose a value
-        Q.current <- Q.temp[beg[r]:fin[r], beg[r]:fin[r]]
-        block.var <- chol2inv(chol(Q.current))
-        block.mean <- - block.var %*% Q.temp[beg[r]:fin[r], -(beg[r]:fin[r])] %*% phi[-(beg[r]:fin[r])]
-        proposal.phi[beg[r]:fin[r]] <- mvrnorm(n=1, mu=phi[beg[r]:fin[r]], Sigma=(proposal.sd.phi * block.var))
-        logit.proposal <- beta.offset[beg[r]:fin[r]] + proposal.phi[beg[r]:fin[r]]
-        logit.current <- beta.offset[beg[r]:fin[r]] + phi[beg[r]:fin[r]]    
+        Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
+        lambda <- rho *  Wstar.list.values[[r]] + (1-rho)  
+        block.var <- tau2 * Wstar.list.vectors[[r]]  %*% diag(1/lambda) %*% t(Wstar.list.vectors[[r]])
+        block.mean <- - block.var %*% (Q.temp[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])] %*% phi[-(phi.beg[r]:phi.fin[r])])
+        proposal.phi <- phi[phi.beg[r]:phi.fin[r]] + (sqrt(proposal.sd.phi) * t(chol(block.var))) %*% b[phi.beg[r]:phi.fin[r]]             
+        logit.proposal <- beta.offset[phi.beg[r]:phi.fin[r]] + proposal.phi
+        logit.current <- beta.offset[phi.beg[r]:phi.fin[r]] + phi[phi.beg[r]:phi.fin[r]]    
         prob.proposal <- exp(logit.proposal)  / (1 + exp(logit.proposal))
         prob.current <- exp(logit.current)  / (1 + exp(logit.current))
         
         ## Calculate the acceptance probability
-        prob1 <- sum(Y[beg[r]:fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[beg[r]:fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
-        prob2 <- t(phi[beg[r]:fin[r]] - block.mean) %*% Q.current %*% (phi[beg[r]:fin[r]] - block.mean) - t(proposal.phi[beg[r]:fin[r]] - block.mean) %*% Q.current %*% (proposal.phi[beg[r]:fin[r]] - block.mean)
+        prob1 <- sum(Y[phi.beg[r]:phi.fin[r]] * (log(prob.proposal) - log(prob.current)) + failures[phi.beg[r]:phi.fin[r]] * (log(1-prob.proposal) - log(1-prob.current)))          
+        prob2 <- t(phi[phi.beg[r]:phi.fin[r]] - block.mean) %*% Q.current %*% (phi[phi.beg[r]:phi.fin[r]] - block.mean) - t(proposal.phi - block.mean) %*% Q.current %*% (proposal.phi - block.mean)
         prob <- exp(prob1 + 0.5 * prob2)
 
         ## Accept or reject the value
             if(prob > runif(1))
             {
-            phi[beg[r]:fin[r]] <- proposal.phi[beg[r]:fin[r]]
+            phi[phi.beg[r]:phi.fin[r]] <- proposal.phi
             accept[3] <- accept[3] + 1  
-            accept[4] <- accept[4] + 1 
             }else
             {
-            proposal.phi[beg[r]:fin[r]] <- phi[beg[r]:fin[r]]
-            accept[4] <- accept[4] + 1 
             }
-        }
-        
-    phi <- phi - mean(phi)
-    
-    
+        }                      
+     accept[4] <- accept[4] + n.phi.block
+     phi <- phi - mean(phi)
+
+
 
     ##################
     ## Sample from tau2
     ##################
-    tau2.posterior.scale <- 0.5 * t(phi) %*% Q %*% phi
-    tau2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=tau2.posterior.scale)
-            while(tau2 > prior.max.tau2)
-            {
-            tau2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=tau2.posterior.scale)
-            }
+    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
+    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
     
     
@@ -352,33 +361,26 @@ failures <- trials - Y
     ## Sample from rho
     ##################
     #### Propose a value
-    proposal.rho <- rnorm(n=1, mean=rho, sd=proposal.sd.rho)
-            while(proposal.rho >= 1 | proposal.rho < 0)
-            {
-            proposal.rho <- rnorm(n=1, mean=rho, sd=proposal.sd.rho)
-            }      
-   
+    proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1,  mean=rho, sd=proposal.sd.rho)    
+    
     #### Calculate the acceptance probability
     proposal.Q <- -proposal.rho * W 
     diag(proposal.Q) <- n.neighbours * proposal.rho + 1 - proposal.rho
-    proposal.det.Q <- 0.5 * as.numeric(determinant(proposal.Q, logarithm=TRUE))[1]
+    proposal.det.Q <- 0.5 * sum(log((proposal.rho * diag(Wstar.val) + (1-proposal.rho))))
     logprob.current <- det.Q - tau2.posterior.scale / tau2
-    logprob.proposal <- proposal.det.Q - 0.5 * t(phi) %*% proposal.Q %*% phi / tau2
+    logprob.proposal <- proposal.det.Q - 0.5 * sum(phi * (proposal.Q %*% phi)) / tau2
     prob <- exp(logprob.proposal - logprob.current)
     
     #### Accept or reject the proposal
-            if(prob > runif(1))
-            {
-            rho <- proposal.rho
-            Q <- proposal.Q
-            det.Q <- proposal.det.Q        
-            accept[5] <- accept[5] + 1  
-            accept[6] <- accept[6] + 1 
-            }else
-            {
-            accept[6] <- accept[6] + 1 
-            }
-
+         if(prob > runif(1))
+         {
+         rho <- proposal.rho
+         Q <- proposal.Q
+         det.Q <- proposal.det.Q        
+         }else
+         {
+         }              
+    
 
    
     #########################
@@ -386,7 +388,7 @@ failures <- trials - Y
     #########################
     logit <- as.numeric(X.standardised %*% beta) + phi + offset    
     prob <- exp(logit)  / (1 + exp(logit))
-	 deviance <- -2 * sum(dbinom(x=Y, size=trials, prob=prob, log=TRUE))
+    deviance <- -2 * sum(dbinom(x=Y, size=trials, prob=prob, log=TRUE))
 
 
 
@@ -416,7 +418,7 @@ failures <- trials - Y
         accept.beta <- 100 * accept[1] / accept[2]
         accept.phi <- 100 * accept[3] / accept[4]
         accept.all <- accept.all + accept
-        accept <- c(0,0,0,0,0,0)
+        accept <- c(0,0,0,0)
             
         #### beta tuning parameter
             if(accept.beta > 40)
