@@ -28,7 +28,7 @@ diag(cor.X) <- 0
  
  	 if(p>1)
 	 {
-    	if(sort(apply(X, 2, sd))[2]==0) stop("the covariate matrix has two intercept terms.", call.=FALSE)
+    	 if(sort(apply(X, 2, sd))[2]==0) stop("the covariate matrix has two intercept terms.", call.=FALSE)
 	 }else
 	 {
 	 }
@@ -80,7 +80,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 
 #### Initial parameter values
 ## Regression parameters beta
-	 if(is.null(beta)) beta <- lm(Y~X.standardised-1, offset=offset)$coefficients
+    if(is.null(beta)) beta <- lm(Y~X.standardised-1, offset=offset)$coefficients
     if(length(beta)!= p) stop("beta is the wrong length.", call.=FALSE)
     if(sum(is.na(beta))>0) stop("beta has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(beta)) stop("beta has non-numeric values.", call.=FALSE)
@@ -126,6 +126,38 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(blocksize.phi <= 0) stop("blocksize.phi is less than or equal to zero", call.=FALSE)
     if(!(floor(blocksize.phi)==ceiling(blocksize.phi))) stop("blocksize.phi has non-integer values.", call.=FALSE)
 
+
+## Compute the blocking structure for phi
+     if(blocksize.phi >= n)
+     {
+     n.phi.block <- 1
+     phi.beg <- 1
+     phi.fin <- n  
+     }else
+     {
+     n.standard <- 1 + floor((n-blocksize.phi) / blocksize.phi)
+     remainder <- n - (n.standard * blocksize.phi)
+     
+          if(remainder==0)
+          {
+          phi.beg <- c(1,seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi))
+          n.phi.block <- length(phi.beg)
+          }else if(remainder==1)
+          {
+          phi.beg <- c(1, seq((blocksize.phi), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi-1, seq((blocksize.phi+blocksize.phi-1), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)    
+          }else
+          {
+          phi.beg <- c(1, seq((blocksize.phi+1), n, blocksize.phi))
+          phi.fin <- c(blocksize.phi, seq((blocksize.phi+blocksize.phi), n, blocksize.phi), n)
+          n.phi.block <- length(phi.beg)
+          }
+     }
+
+
+
 ## Matrices to store samples
 samples.beta <- array(NA, c((n.sample-burnin), p))
 samples.phi <- array(NA, c((n.sample-burnin), n))
@@ -135,8 +167,9 @@ samples.rho <- array(NA, c((n.sample-burnin), 1))
 samples.deviance <- array(NA, c((n.sample-burnin), 1))
 
 ## Metropolis quantities
-accept <- rep(0,2)
-proposal.sd.rho <- 0.05
+proposal.sd.rho <- 0.02
+nu2.posterior.shape <- 0.5*n-1
+tau2.posterior.shape <- 0.5*n-1
 
 
 #### Priors
@@ -178,9 +211,13 @@ proposal.sd.rho <- 0.05
     if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
     if(!sum(names(table(W))==c(0,1))==2) stop("W has non-binary (zero and one) values.", call.=FALSE)
 
+
 n.neighbours <- as.numeric(apply(W, 1, sum))
 Q <- diag(n.neighbours) - rho * W
-det.Q <- 0.5 * as.numeric(determinant(Q, logarithm=TRUE))[1]
+spam.Q <- as.spam(Q)
+spam.Q.proposal <- spam.Q
+det.Q <- sum(log(diag(chol.spam(spam.Q))))
+indices.Q <- which(as.numeric(Q)!=0)
 
 
 #### Beta update quantities
@@ -196,6 +233,7 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
 	}
 
 
+
 ###########################
 #### Run the Bayesian model
 ###########################
@@ -204,13 +242,11 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
     ####################
     ## Sample from beta
     ####################
-    #### Calculate the full conditional mean and variance
     data.mean.beta <- data.temp.beta %*% (Y - phi - offset)
-    fc.variance.beta <- solve((prior.precision.beta + data.precision.beta / nu2))
-    fc.mean.beta <- fc.variance.beta %*% (prior.precision.beta %*% prior.mean.beta + (data.precision.beta / nu2) %*% data.mean.beta)
-    
-    #### Update beta by Gibbs sampling  
-    beta <- mvrnorm(n=1, mu=fc.mean.beta, Sigma=fc.variance.beta)
+    U <- chol((prior.precision.beta + data.precision.beta / nu2))
+    Uinv <- backsolve(U, diag(rep(1,p)))
+    fc.mean.beta <- Uinv %*% (t(Uinv) %*% (prior.precision.beta %*% prior.mean.beta + (data.precision.beta / nu2) %*% data.mean.beta))
+    beta <- fc.mean.beta + Uinv %*% rnorm(p)
     
     
     
@@ -218,82 +254,45 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
     ## Sample from nu2
     ##################
     fitted.current <-  as.numeric(X.standardised %*% beta) + phi + offset
-	 nu2.posterior.scale <- 0.5 * sum((Y - fitted.current)^2)
-    nu2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=nu2.posterior.scale)
-            while(nu2 > prior.max.nu2)
-            {
-            nu2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=nu2.posterior.scale)
-            }
-
+    nu2.posterior.scale <- 0.5 * sum((Y - fitted.current)^2)
+    nu2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.nu2), b=Inf,  shape=nu2.posterior.shape, scale=(1/nu2.posterior.scale))
+    
 
 
     ####################
     ## Sample from phi
     ####################
-    #### Create the blocking structure
-    if(blocksize.phi >= n)
-    {
-    n.block <- 1
-    beg <- 1
-    fin <- n
-    }else
-    {
-    init <- sample(1:blocksize.phi,  1)
-    n.standard <- floor((n-init) / blocksize.phi)
-    remainder <- n - (init + n.standard * blocksize.phi)
-        
-        if(n.standard==0)
-        {
-        beg <- c(1,(init+1))
-        fin <- c(init,n)
-        }else if(remainder==0)
-        {
-        beg <- c(1,seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi))
-        }else
-        {
-        beg <- c(1, seq((init+1), n, blocksize.phi))
-        fin <- c(init, seq((init+blocksize.phi), n, blocksize.phi), n)
-        }
-    n.block <- length(beg)
-    }
-    
-
-    #### Update the parameters in blocks
     Q.temp <- Q / tau2
     data.mean.phi <- Y - as.numeric(X.standardised %*% beta) - offset    
-        
-        for(r in 1:n.block)
-        {
-        ## Create the prior and data means and variances
-        prior.precision.phi <- as.matrix(Q.temp[beg[r]:fin[r], beg[r]:fin[r]])
-        block.length <- nrow(prior.precision.phi)
-        prior.var.phi <- chol2inv(chol(prior.precision.phi))
-        prior.mean.phi <- - prior.var.phi %*% Q.temp[beg[r]:fin[r], -(beg[r]:fin[r])] %*% phi[-(beg[r]:fin[r])]
-        data.precision.phi <- diag(rep((1/nu2),(block.length+1)))
-		  data.precision.phi <- data.precision.phi[1:block.length, 1:block.length]
-
-        ## Create the full conditional
-        fc.variance.phi <- solve((prior.precision.phi + data.precision.phi))
-        fc.mean.phi <- fc.variance.phi %*% (prior.precision.phi %*% prior.mean.phi + data.precision.phi %*% data.mean.phi[beg[r]:fin[r]])
-        
-        ## Update phi
-        phi[beg[r]:fin[r]] <- mvrnorm(n=1, mu=fc.mean.phi, Sigma=fc.variance.phi)
-		  }
-                
-    phi <- phi - mean(phi)
+    data.precision.phi <- diag(rep((1/nu2), blocksize.phi))
+    b <- rnorm(n)
     
+         for(r in 1:n.phi.block)
+         {
+         ## Create the prior  mean and variance
+         n.current <- length(phi.beg[r]:phi.fin[r])     
+         Q.current <- Q.temp[phi.beg[r]:phi.fin[r], phi.beg[r]:phi.fin[r]]
+         U <- chol(Q.current)
+         Uinv <- backsolve(U, diag(rep(1,n.current)))
+         block.mean <- - Uinv %*% (t(Uinv) %*% Q.temp[phi.beg[r]:phi.fin[r], -(phi.beg[r]:phi.fin[r])] %*% phi[-(phi.beg[r]:phi.fin[r])])
+          
+         ## Create the full conditional
+         U2 <- chol((Q.current + data.precision.phi[1:n.current, 1:n.current])) 
+         U2inv <- backsolve(U2, diag(rep(1,n.current)))
+         fc.mean.phi <- U2inv %*% t(U2inv) %*% (Q.current %*% block.mean + data.precision.phi[1:n.current, 1:n.current] %*% data.mean.phi[phi.beg[r]:phi.fin[r]])
+         
+         ## Update phi
+         phi[phi.beg[r]:phi.fin[r]] <- fc.mean.phi + U2inv %*% b[phi.beg[r]:phi.fin[r]]
+        }   
+    phi <- phi - mean(phi)
+
     
 
     ##################
     ## Sample from tau2
     ##################
-    tau2.posterior.scale <- 0.5 * t(phi) %*% Q %*% phi
-    tau2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=tau2.posterior.scale)
-            while(tau2 > prior.max.tau2)
-            {
-            tau2 <- rinvgamma(n=1, shape=(0.5*n-1), scale=tau2.posterior.scale)
-            }
+    tau2.posterior.scale <- 0.5 * sum(phi * (Q %*% phi))
+    tau2 <- 1/rtrunc(n=1, spec="gamma", a=(1/prior.max.tau2), b=Inf,  shape=tau2.posterior.shape, scale=(1/tau2.posterior.scale))
     
     
     
@@ -301,33 +300,29 @@ data.temp.beta <- data.var.beta %*% t(X.standardised)
     ## Sample from rho
     ##################
     #### Propose a value
-    proposal.rho <- rnorm(n=1, mean=rho, sd=proposal.sd.rho)
-            while(proposal.rho >= 1 | proposal.rho < 0)
-            {
-            proposal.rho <- rnorm(n=1, mean=rho, sd=proposal.sd.rho)
-            }      
-   
+    proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1,  mean=rho, sd=proposal.sd.rho)    
+    
     #### Calculate the acceptance probability
     proposal.Q <- diag(n.neighbours) - proposal.rho * W
-    proposal.det.Q <- 0.5 * as.numeric(determinant(proposal.Q, logarithm=TRUE))[1]
+    spam.Q.proposal@entries <- as.numeric(proposal.Q)[indices.Q]   
+    proposal.det.Q <- sum(log(diag(chol.spam(spam.Q.proposal))))
     logprob.current <- det.Q - tau2.posterior.scale / tau2
-    logprob.proposal <- proposal.det.Q - 0.5 * t(phi) %*% proposal.Q %*% phi / tau2
+    logprob.proposal <- proposal.det.Q - 0.5 * sum(phi * (spam.Q.proposal %*% phi)) / tau2
     prob <- exp(logprob.proposal - logprob.current)
     
     #### Accept or reject the proposal
-            if(prob > runif(1))
-            {
-            rho <- proposal.rho
-            Q <- proposal.Q
-            det.Q <- proposal.det.Q         
-            accept[1] <- accept[1] + 1  
-            accept[2] <- accept[2] + 1 
-            }else
-            {
-            accept[2] <- accept[2] + 1 
-            }
-
+         if(prob > runif(1))
+         {
+         rho <- proposal.rho
+         Q <- proposal.Q
+         spam.Q <- spam.Q.proposal
+         det.Q <- proposal.det.Q        
+         }else
+         {
+         }
+    
    
+    
     #########################
     ## Calculate the deviance
     #########################
